if (debug) {
	console.log("atlas_ui.js loaded")
}

var atlasRegions = [];
var atlasServerApiUrl = "http://medpc076.ime.kfa-juelich.de:5000/api"
// var serverHost = "medpc076.ime.kfa-juelich.de"
// var serverPort = 80
// var serverEndpoint = "atlas_server/"
// var atlasId = null;

function JobConfiguration(account, time, job_type, gres, mail_type, mail_user, num_nodes, partition, reservation) {
	this.account = account;
	this.time = time;
	this.job_type = job_type;
	this.gres = gres;
	this.mail_type = mail_type;
	this.mail_user = mail_user;
	this.num_nodes = num_nodes;
	this.partition = partition;
	this.reservation = reservation;
}

function Annotation(name, section, path) {
	this.name = name;
	this.section = section;
	this.path = path;
}

function Project() {
	this.brain = 20;
	this.name = "Untitled"
	this.owner = myOrigin.user;
	this.tasks = [];
}

function Task() {
	this.name = "Untitled";
	this.annotations = [];
	this.jobs = [];
}

function compareRegions(region0, region1) {
	if (region0 === region1) {
		return true;
	}
	if (region0.id === region1.id) {
		return true;
	}
	return false;
}

function searchRegionInList(region, list) {
	for (var i in list) {
		if (compareRegions(list[i], region)) {
			return i;
		}
	}
	return -1;
}

function getTimestamp() {
	var date = new Date();
	return `${date.toLocaleDateString()} ${date.toLocaleTimeString()}`;
}

function post_message(msg) {
	var msg_box = $("#messages");
	var timestamp = getTimestamp();
	// msg_box.append(`<i>${timestamp}</i><br>${msg}<br>`);
	console.log(`${timestamp} ${msg}`);
}

function removeRegionFromList(region, list) {
	let index = searchRegionInList(region, list);
	if (index != -1) {
		list.splice(index, 1);
	}
}

var app = new Vue({
	el: '#atlas_app',
	data: {
		brain: 20,
		available_annotations: [],
		// atlas_id: null,
		projects: [],
		selected_project: null,
		selected_task: null,
		selected_job: null,
		enable_auto_rename: true,
	},
	methods: {
		getProjects: function () {
			// Get username
			let user = undefined;
			if (typeof test_user !== "undefined") {
				user = test_user;
			}
			else {
				user = myOrigin.user;
			}
			// let user = myOrigin.user;
			// Request projects from server
			let endpoint = `${atlasServerApiUrl}/projects/users/${user}`;
			let _this = this;
			// let result =
			$.ajax({
				url: endpoint,
				type: "GET",
				crossDomain: true,
				success: function (data) {
					_this.projects = data;
					if (_this.projects.length > 0) {
						_this.selected_project = _this.projects[0];
					}
					post_message(`Got ${_this.projects.length} projects from server.`);
				},
				error: function (jqXHR, textStatus, errorThrown) {
					post_message(`Error getting projects: ${textStatus} ${errorThrown}`);
				}
			});
		},
		initializeAtlas: function () {
			// this.checkOrGetAtlasId();
			this.getProjects();
		},
		updateSelectedProject: function (project) {
			if (project !== this.selected_project) {
				this.selected_project = project;
				this.selected_task = null;
			}
		},
		updateSelectedTask: function (task) {
			if (task !== this.selected_task) {
				this.selected_task = task;
			}
		},
		updateSelectedJob: function (job) {
			if (job !== this.selected_job) {
				this.selected_job = job;
			}
		},
		showJobsForTask: function (project, task) {
			this.selected_task = task;
			this.getJobStatusForAll(project, task);
		},
		newProject: function () {
			// Create a new project and set it as selected project to be edited
			let new_project = new Project();
			this.selected_project = new_project;
		},
		editProject: function (project) {
			// To edit a project, we set it as selected project
			this.selected_project = project;
		},
		saveProject: function (project) {
			if (project.created === undefined) {
				// This is a new project, so we add it to the project list
				// and send it to the server
				let endpoint = `${atlasServerApiUrl}/projects`;
				let _this = this;
				$.ajax({
					url: endpoint,
					type: "POST",
					dataType: "json",
					contentType: "application/json",
					data: JSON.stringify(project),
					crossDomain: true,
					success: function (data) {
						let new_project = data;
						post_message(`Created project ${new_project.name} (id ${new_project.project_id})`);
						_this.projects.push(new_project);
						_this.closeProject();
					},
					error: function (jqXHR, textStatus, errorThrown) {
						post_message(`Error creating project: ${textStatus} ${errorThrown}`);
						_this.closeProject();
					}
				});
			} else {
				// This is an existing project, so we update it
				let endpoint = `${atlasServerApiUrl}/projects/${project.project_id}`;
				let _this = this;
				$.ajax({
					url: endpoint,
					type: "PUT",
					dataType: "json",
					contentType: "application/json",
					data: JSON.stringify(project),
					crossDomain: true,
					success: function (data) {
						let updated_project = data;
						post_message(`Updated project ${updated_project.name} (id ${updated_project.project_id})`);
						// Find index of project to update it
						for (var i in _this.projects) {
							if (_this.projects[i] === project) {
								// Replace object, using a vue function so vue detects the change
								Vue.set(_this.projects, i, updated_project);
								break;
							}
						}
						_this.closeProject();
					},
					error: function (jqXHR, textStatus, errorThrown) {
						post_message(`Error updating project: ${textStatus} ${errorThrown}`);
						_this.closeProject();
					}
				});
			}
		},
		deleteProject: function (project_index) {
			let project = this.projects[project_index];

			// Confirm delete
			let confirm_delete = confirm(`Do you want to permanently remove project ${project.project_id} - "${project.name}"?`)
			if (!confirm_delete) {
				return;
			}

			console.log(`Delete: ${project.project_id}`);
			// Remove the project from the list
			this.projects.splice(project_index, 1);

			if (project.created === undefined) {
				// Not yet in the database, so no need to remove
				return;
			}

			let endpoint = `${atlasServerApiUrl}/projects/${project.project_id}`;
			$.ajax({
				url: endpoint,
				type: "DELETE",
				crossDomain: true,
				success: function (data) {
					post_message(`Removed project ${project.name} (id ${project.project_id})`);
				},
				error: function (jqXHR, textStatus, errorThrown) {
					post_message(`Error removing project: ${textStatus} ${errorThrown}`);
				}
			});
		},
		closeProject() {
			this.selected_project = null;
			this.selected_task = null;
		},
		newTask: function () {
			// Create a new task and set it as selected task to be edited
			let new_task = new Task();
			this.selected_task = new_task;
			// Refresh annotations
			this.updateAvailableAnnotations();
		},
		editTask: function (task) {
			// To edit a task, we set it as selected task
			this.selected_task = task;
			// Refresh annotations
			this.updateAvailableAnnotations();
		},
		saveTask: function (project, task) {
			let _this = this;
			if (task.created === undefined) {
				if (project.created === undefined) {
					// If the project is also new, we don't save the tasks here, but with the new project
					post_message(`Delaying saving of new task since project has not been saved yet.`)
					project.tasks.push(task);
				} else {
					// Add the new task to the project
					let endpoint = `${atlasServerApiUrl}/projects/${project.project_id}/tasks`;
					$.ajax({
						url: endpoint,
						type: "POST",
						dataType: "json",
						contentType: "application/json",
						data: JSON.stringify(task),
						crossDomain: true,
						success: function (data) {
							let created_task = data;
							post_message(`Created task ${created_task.name} (id ${project.project_id} - ${created_task.task_id})`);
							// Add the task to the existing project
							project.tasks.push(created_task);
							_this.closeTask();
						},
						error: function (jqXHR, textStatus, errorThrown) {
							post_message(`Error creating task: ${textStatus} ${errorThrown}`);
							_this.closeTask();
						}
					});
				}
			} else {
				// The task exists, update it
				let endpoint = `${atlasServerApiUrl}/projects/${project.project_id}/tasks/${task.task_id}`;
				$.ajax({
					url: endpoint,
					type: "PUT",
					dataType: "json",
					contentType: "application/json",
					data: JSON.stringify(task),
					crossDomain: true,
					success: function (data) {
						let updated_task = data;
						post_message(`Updated task ${updated_task.name} (id ${project.project_id} - ${updated_task.task_id})`);
						// Find the corresponding task in the project and update it
						for (var i in project.tasks) {
							if (project.tasks[i] === task) {
								// Replace object, using a vue function so vue detects the change
								Vue.set(project.tasks, i, updated_task);
								break;
							}
						}
						_this.closeTask();
					},
					error: function (jqXHR, textStatus, errorThrown) {
						post_message(`Error updating task: ${textStatus} ${errorThrown}`);
						_this.closeTask();
					}
				});

			}
		},
		addTask: function () {
			let new_task = new Task();
			this.selected_project.tasks.push(new_task);
			this.updateSelectedTask(new_task);
		},
		addAnnotationToTask: function (index) {
			let annotation_to_add = this.available_annotations[index];
			this.selected_task.annotations.push(annotation_to_add);
			if (this.enable_auto_rename) {
				// Save original name, so we can restore it later
				let original_name = annotation_to_add.name;
				annotation_to_add.original_name = original_name;
				let pattern = /_(l|r)(_\d+)?$/i
				annotation_to_add.name = annotation_to_add.name.replace(pattern, "");
			}
			this.available_annotations.splice(index, 1);
		},
		removeAnnotationFromTask: function (index) {
			let annoation_to_remove = this.selected_task.annotations[index];
			this.selected_task.annotations.splice(index, 1);
			if (annoation_to_remove.original_name !== undefined) {
				// Restore original name
				annoation_to_remove.name = annoation_to_remove.original_name;
			}
			this.available_annotations.push(annoation_to_remove);
		},
		updateAvailableAnnotations: function () {
			// Clear available regions
			this.available_annotations.splice(0, this.available_annotations.length);
			// Add current regions
			var regions = ImageInfo[currentImage]["Regions"];
			for (var i in regions) {
				let region = regions[i];
				// Important: Temporarily deselect region before exporting JSON, as selected regions have a different export format...
				let selected_state = region.path.selected;
				region.path.selected = false;
				let new_annotation = new Annotation(region.name, currentImage, region.path.exportJSON());
				region.path.selected = selected_state;
				this.available_annotations.push(new_annotation);
			}
		},
		deleteTask(project, task_index) {
			// Delete the task from the project
			let task = project.tasks[task_index];

			// Confirm delete
			let confirm_delete = confirm(`Do you want to permanently remove task ${task.task_id} - "${task.name}"?`)
			if (!confirm_delete) {
				return;
			}

			project.tasks.splice(task_index, 1);
			// Delete task from project
			let endpoint = `${atlasServerApiUrl}/projects/${project.project_id}/tasks/${task.task_id}`;
			$.ajax({
				url: endpoint,
				type: "DELETE",
				crossDomain: true,
				success: function (data) {
					// Remove task from the project
					project.tasks.splice(task_index, 1);
					post_message(`Deleted task task ${task.name} (id ${project.project_id} - ${task.task_id})`);
				},
				error: function (jqXHR, textStatus, errorThrown) {
					post_message(`Error deleting task: ${textStatus} ${errorThrown}`);
				}
			});

		},
		closeTask() {
			this.selected_task = null;
		},
		submitJob(project, task, job_configuration) {
			let endpoint = `${atlasServerApiUrl}/projects/${project.project_id}/tasks/${task.task_id}/jobs`;

			let _this = this;
			$.ajax({
				url: endpoint,
				type: "POST",
				dataType: "json",
				contentType: "application/json",
				data: JSON.stringify(job_configuration),
				crossDomain: true,
				success: function (data) {
					let job = data;
					post_message(`Submitted job ${job.job_id}`);
					task.jobs.push(job);
					_this.getJobStatus(project, task, job);
				},
				error: function (jqXHR, textStatus, errorThrown) {
					let msg = `Error submitting job. Server may be unreachable.\nStatus: ${textStatus}\nMessage: ${errorThrown}`;
					alert(msg);
					post_message(msg);
				}
			});
		},
		submitTrainingJob(project, task) {
			let job_configuration = new JobConfiguration(
				// account
				// "jinm14",
				"jinm16",
				// time
				"01:00:00",
				// job_type
				"training",
				// gres
				// "",
				"gpu:4",
				// mail_type
				"FAIL",
				// mail_user
				"c.schiffer@fz-juelich.de",
				// num_nodes
				1,
				// partition
				// "devel",
				"dc-gpu",
				//  reservation
				"",
			);
			this.submitJob(project, task, job_configuration);
		},
		submitPredictionJob(project, task) {
			let job_configuration = new JobConfiguration(
				// account
				// "jinm14",
				"jinm16",
				// time
				"01:00:00",
				// job_type
				"prediction",
				// gres
				// "",
				"gpu:4",
				// mail_type
				"FAIL",
				// mail_user
				"c.schiffer@fz-juelich.de",
				// num_nodes
				1,
				// partition
				// "devel",
				"dc-gpu",
				//  reservation
				"",
			);
			this.submitJob(project, task, job_configuration);
		},
		getJobStatus(project, task, job) {
			let endpoint = `${atlasServerApiUrl}/projects/${project.project_id}/tasks/${task.task_id}/jobs/${job.job_id}`;
			$.ajax({
				url: endpoint,
				type: "GET",
				crossDomain: true,
				success: function (data) {
					// Update job state
					post_message(`Updated status of job ${job.job_id}`);
					if (data.status.JobState) {
						job.status = data.status.JobState;
					} else {
						// If the status cannot be determined, assume it is completed
						job.status = "UNKNOWN";
					}
				},
				error: function (jqXHR, textStatus, errorThrown) {
					post_message(`Error getting status job status: ${textStatus} ${errorThrown}`);
				}
			});
		},
		cancelJob: function (project, task, job_index) {
			// Cancel the given job
			let job = task.jobs[job_index]
			let endpoint = `${atlasServerApiUrl}/projects/${project.project_id}/tasks/${task.task_id}/jobs/${job.job_id}`;
			$.ajax({
				url: endpoint,
				type: "DELETE",
				crossDomain: true,
				success: function (data) {
					// Remove job from task list and save it
					task.jobs.splice(job_index, 1);
					post_message(`Deleted job ${job.job_id} of task ${task.name} (id ${project.project_id} - ${task.task_id})`);
				},
				error: function (jqXHR, textStatus, errorThrown) {
					post_message(`Error canceling job: ${textStatus} ${errorThrown}`);
				}
			});
		},
		cancelAllJobs: function (project, task) {
			// Cancel all jobs for a task
			for (var job_index in task.jobs) {
				this.cancelJob(project, task, job_index);
			}
		},
		getJobStatusForAll: function (project, task) {
			for (var i in task.jobs) {
				this.getJobStatus(project, task, task.jobs[i]);
			}
		},
		pollJobStatus: function () {
			if (this.selected_project !== null && this.selected_task !== null) {
				post_message(`Polling for job status of project ${this.selected_project.project_id}, task ${this.selected_task.task_id}`);
				this.getJobStatusForAll(this.selected_project, this.selected_task);
			}
		},
		getResultLink: function (project, task) {
			let host = window.location.host;
			let path = window.location.pathname;
			let protocol = window.location.protocol;
			let directory = path.substring(0, path.lastIndexOf("/"));
			let source = params.source;
			// Note: This should match the format atlas_server uses for storing the overlays"
			let overlay = `${project.name} - ${task.name}`

			let target = `${protocol}//${host}${directory}/${source}`
			let encoded_target = encodeURIComponent(btoa(target));

			let section = task.annotations[0].section;

			let endpoint = `${atlasServerApiUrl}/projects/${project.project_id}/tasks/${task.task_id}/tile_server_config/${encoded_target}`;
			let target_link = `${protocol}//${host}${path}?overlay=${overlay}&opacity=0.4&%sliceNumber=${section}&source=${endpoint}`

			window.location.href = target_link;
		}
	},
	created: function () {
		post_message("Register job status polling")
		setInterval(function () {
			this.pollJobStatus();
		}.bind(this), 5000);
	},
});
